/*
* Copyright (C) 2007 Cygnus <cygnus2 at post.cz>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
* USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VERSION			"0.1"
#define BIN_BYTES_PER_LINE	6
#define HEX_BYTES_PER_LINE	16
#define CHARS_PER_LINE		32

// globln parametry - konfigurace programu
volatile int	verbose = 1;
volatile int	addresses = 0;

enum {bin = 0, hex = 1, txt = 2} style;
char hexn [16] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

int convert (char* filename, unsigned int output_style)
{
	FILE *file;
	unsigned char byte_from_file;
	unsigned char byte;

	int byte_counter;
	int byte_line_counter;

	unsigned char bit;
	char bin_byte [9] = "00000000\0";

	char hex_byte [3] = "00\0";

	file = fopen (filename,"r");
	if (file == NULL)
	{
		printf ("Error: Can't open source file %s.\n", filename);
		exit (-1);
	}

	byte_counter = 0;
	byte_line_counter = 0;

	while (1)
	{
		if (fread (&byte_from_file, sizeof (unsigned char), 1, file) != 1)
		{
			if ((output_style == txt) && (byte_counter > 0)) printf ("\"\n");
			printf ("\n\nEnd of file 1\n");
			break;
		}

		// beginning of new line
		if (byte_line_counter == 0)
		{
			if (addresses) printf ("A_%i", byte_counter);
			switch (output_style)
			{

				case bin:
				case hex:
					printf ("\t\tdb\t");
					break;
				case txt:
					printf ("\t\tdb\t\"");
					break;
			}
		}

		// convert byte from file to correct text form (ASL compatible)
		switch (output_style)
		{
			case bin:
				byte = byte_from_file;

				for (bit = 0; bit < 8; bit++)
				{
					if (byte == 2 * (byte / 2)) bin_byte[7 - bit] = '0'; else bin_byte[7 - bit] = '1';
					byte = byte / 2;
				}

				if (byte_line_counter > 0) printf (", ");
				printf ("%sb", (char *) &bin_byte);
				break;
			case hex:
				byte = byte_from_file / 16;

				hex_byte [1] = hexn [byte];
				hex_byte [0] = hexn [byte_from_file - 16 * byte];

				if (byte_line_counter > 0) printf (", ");
				printf ("%c%ch", hex_byte[1], hex_byte[0]);
				break;
			case txt:
				byte = byte_from_file;

				//if (byte > 127) byte = byte & 127;
				if (byte > 127) byte = '.';
				if (byte < 32) byte = 32;

				printf ("%c", (char) byte);
		}

		byte_counter++;
		byte_line_counter++;

		// detect end of line for all cases
		switch (output_style)
		{
			case bin:
				if (byte_line_counter > BIN_BYTES_PER_LINE - 1)
				{
					byte_line_counter = 0;
					printf ("\n");
				}
				break;
			case hex:
				if (byte_line_counter > HEX_BYTES_PER_LINE - 1)
				{
					byte_line_counter = 0;
					printf ("\n");
				}
				break;
			case txt:
				if (byte_line_counter > CHARS_PER_LINE - 1)
				{
					byte_line_counter = 0;
					printf ("\"\n");
				}
				break;
		}
	}
}

void version ()
{
	printf ("%s \n", VERSION);
}

void help ()
{
	printf ("Usage: binary2source [OPTIONS] [FILENAME]\n");
	printf ("  -h, --help       this text\n");
	printf ("  -a, --address    print relative addresses\n");
	printf ("  -b, --binary     output as binary define bytes\n");
	printf ("  -H, --hex        output as hexadecimal define bytes\n");
	printf ("  -t, --text       only ascii! other chars will modified\n");
	printf ("  -v, --version    version\n");
	printf ("\n");
	printf ("Output text can be used as source for ASL compiler.\n");
	printf ("\n");
	printf ("Don't use more than one parameter, that defines output\n");
	printf ("style. When more output style parameters will used, all\n");
	printf ("except last will ignored.\n");
}

int main(int argc, char *argv[])
{
	char *filename;
	unsigned int counter;
	unsigned int switches;
	unsigned int correct;
	unsigned int output_style;

	switches = 1;
	addresses = 0;

	for (counter=1; counter<argc; counter++)
	{
		correct = 0;

		if (!strcmp(argv[counter], "-h") || (!strcmp(argv[counter], "--help")))
		{
			help ();
			switches++;
			correct = 1;
			return (0);
		}
		if (!strcmp(argv[counter], "-a") || (!strcmp(argv[counter], "--address")))
		{
			addresses = 1;
			switches++;
			correct = 1;
		}
		if (!strcmp(argv[counter], "-v") || (!strcmp(argv[counter], "--version")))
		{
			version ();
			switches++;
			correct = 1;
			return (0);
		}
		if (!strcmp(argv[counter], "-b") || (!strcmp(argv[counter], "--binary")))
		{
			output_style = bin;
			switches++;
			correct = 1;
		}
		if (!strcmp(argv[counter], "-H") || (!strcmp(argv[counter], "--hex")))
		{
			output_style = hex;
			switches++;
			correct = 1;
		}
		if (!strcmp(argv[counter], "-t") || (!strcmp(argv[counter], "--text")))
		{
			output_style = txt;
			switches++;
			correct = 1;
		}
		if ((correct==0) && (argv[counter][0] == '-'))
		{
			printf ("ERROR: unknown parametr - \"%s\"\n", argv[counter]);
			help ();
			return (-1);
		}
		if (argv[counter][0] != '-')
		{
			filename=argv[argc - switches - 1];
			convert (filename, output_style);
			return (0);
		}
	}

}

